#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>

#define ALGORITHM				prngXor128
//#define ALGORITHM				prngMwc
#define BLOCKSIZE				4
#define BLOCKCOUNT				4096

/* PRNG algorithm from "Xorshift RNGs", George Marsaglia, The Florida State University */
static uint32_t prngXor128(void) {
	static uint32_t x = 123456789, y = 362436069, z = 521288629, w = 88675123;
	uint32_t t;
	t = (x ^ (x<<11));
	x = y;
	y = z;
	z = w; 
	w = (w ^ (w >> 19)) ^ (t ^ (t >> 8));
	return w;
}

#if 0
/* PRNG algorithm from "Xorshift RNGs", George Marsaglia, The Florida State University */
static uint32_t prngMwc(void) {
	static uint32_t x = 123456789, y = 362436069, z = 77465321, c = 13579;
	uint64_t t;
	t = (uint64_t)916905990 * x + c; 
	x = y; 
	y = z; 
	c = (t >> 32); 
	z = (t & 0xffffffff);
	return z;
}
#endif

int main(int argc, char **argv) {
	if (argc != 2) {
		fprintf(stderr, "%s [Bytecount]\n", argv[0]);
		exit(EXIT_FAILURE);
	}

	uint64_t byteCount = atol(argv[1]);
	uint64_t i;
	for (i = 0; i < byteCount / (BLOCKSIZE * BLOCKCOUNT); i++) {
		uint32_t block[BLOCKCOUNT];
		int j;
		for (j = 0; j < BLOCKCOUNT; j++) {
			block[j] = ALGORITHM();
		}
		fwrite(block, BLOCKCOUNT, BLOCKSIZE, stdout);
	}

	for (i = 0; i < byteCount % (BLOCKSIZE * BLOCKCOUNT) / 4; i++) {
		uint32_t next = ALGORITHM();
		fwrite(&next, 4, 1, stdout);
	}

	{
		uint32_t next = ALGORITHM();
		for (i = 0; i < byteCount % (BLOCKSIZE * BLOCKCOUNT) % 4; i++) {
			fwrite(((unsigned char*)&next) + i, 1, 1, stdout);
		}
	}	

	return 0;
}


